<?php

 
class ChatDialog {

	private $inventor;
	private $tailLength;
	private $dbCmd;
	private $familyTree;

	function __construct($inventorID){
		
		$inventorID = intval($inventorID);
		$this->dbCmd = new DbCmd();
		
		$inventorObj = new Inventor();
		if(!$inventorObj->checkIfExists($inventorID)){
			throw new Exception("The Inventor ID is invalid");
		}
			
		$this->inventor = $inventorID;
		
		$this->tailLength = 100;
		
		$this->familyTree = FamilyTree::getNewGeneration();
		
	}
	

	static function getInventor($dialogID){
		
		$dbCmd = new DbCmd();
		$dbCmd->query("SELECT Inventor FROM chatdialogs WHERE  
							ID=" . intval($dialogID) );
		if($dbCmd->getNumRows() == 0){
			throw new Exception("The dialog ID is invalid");
		}
			
		return $dbCmd->getValue();
	}
	

	function getContentSubmitted($dialogID){
		
		$dialogID = $this->validateDialogID($dialogID);
		
		$this->dbCmd->query("SELECT ContentSubmitted FROM chatdialogs WHERE  
							ID=" . intval($dialogID) );
		return $this->dbCmd->getValue();
	}
	
	function getContentCorrected($dialogID){
		
		$dialogID = $this->validateDialogID($dialogID);
		
		$this->dbCmd->query("SELECT ContentCorrected FROM chatdialogs WHERE  
							ID=" . intval($dialogID) );
		return $this->dbCmd->getValue();
	}
	
	function getContentPredicted($dialogID){
		
		$dialogID = $this->validateDialogID($dialogID);
		
		$this->dbCmd->query("SELECT ContentPredicted FROM chatdialogs WHERE  
							ID=" . intval($dialogID) );
		return $this->dbCmd->getValue();
	}
	
	// After logging in, you will see your ongoing dialog (from previous web sessions).
	// This will set the dialog length which first appears (which may grow large over the years).
	// It allows you to override the default value.
	function setTailLength($amount){
		
		$amount = intval($amount);
		
		if($amount > 10000){
			throw new Exception("The Tail Length can not exceed 10,000");
		}
		
		if($amount < 1){
			throw new Exception("The tail length must be greater than or equal to 1");
		}
				
		$this->tailLength = $amount;
	}
	
	// This returns an array of dialog ID's.
	// The closer the entry is to the front of the array, the newer the thought is.
	// The DialogID at index [0] is the last thought within the inventor's dialog (which could be a response).
	// You can override the length of the array by setting a different "Tail Length"
	function getDialogStream(){
		
		$this->dbCmd->query("SELECT ID FROM chatdialogs
								WHERE Inventor=" . intval($this->inventor) . " 
								ORDER BY ID DESC LIMIT " . $this->tailLength);
		$dialogIDsArr = $this->dbCmd->getValueArr();
		
		return $dialogIDsArr;
	}
	
	// Because this is a chat dialog, we want to get text out after putting text in.
	// Therefore we are going to take the return data (from the Child) and immediately feed it into the Adult.
	// Everyone that types something into the application will effectively be the child... on every single form submission.  
	// The reason is that they have to start off the conversation after creating an account.  Dialog ID #1. Reply #2   Odd=Child... Even=Adult
	// You can think of it as Student/Teacher if you like.
	// The reply that comes back does not actually get saved into the database (as you would think of someone on the other end typing).
	// The reply was already saved to the database in the past.  We are going to reconstruct the Reply by putting in something which was "cleaned up" by the children.
	function addThought($data){
		
		// The person who starts the conversation doesn't get to end it.  That is why we all say "Your Welcome" (in one form or another).
		$this->dbCmd->query("SELECT COUNT(*) FROM chatdialogs WHERE Inventor=" . $this->inventor);
		$inventorDialogCount = $this->dbCmd->getValue();
		
		if(round(($inventorDialogCount % 2), 0) == 0){
			$flipFlopFlag = true;
		}
		else{
			$flipFlopFlag = false;
		}
		
		// This will fix any typos, etc.  You put in something unfamiliar and it will give you back something familar.
		$familiarInput = $this->familyTree->inputRoot($data, $this->inventor, $flipFlopFlag);
		
		// This takes something familiar, and gives you the prediction.
		// It only works because it is being done in pairs (back to back).
		$prediction = $this->familyTree->inputRoot($familiarInput, $this->inventor, $flipFlopFlag);
		
		$i["Inventor"] = $this->inventor;
		$i["Date"] = time();
		$i["ContentSubmitted"] = $data;
		$i["ContentCorrected"] = $familiarInput;
		$i["ContentPredicted"] = $prediction;
		
		return $this->dbCmd->insertQuery("chatdialogs", $i);
	}
	
	// This will add the thought to the DB for training purposes.
	function addOldThought($content){
		
		exit("need to finish this method.");

	}
	
	function formatHtml($data){
		
		$data = WebUtil::htmlOutput($data);
		
		$data = preg_replace('/\n/', "\n<br>\n", $data);
		
		return $data;
	}
	
	private function validateDialogID($dialogID){
		
		$dialogID = intval($dialogID);
		
		$inventorID = self::getInventor($dialogID);
		if($this->inventor != $inventorID){
			throw new Exception("The inventor ID is mismatched to the Dialog ID");
		}
		
		return $dialogID;
	}
	

	

}


