<?php
 
class DbCmd {
	
	
	private $connection;
	private $result;
	private $queryStr;
	private $isDevelopmentServer;
	
	private static $singleton;
	
	// If you don't pass a DbConnection object then it will use the default connection from the "Constants" file.
	function __construct(DbConnect $dbConnectObj = NULL) {

		 if($dbConnectObj == NULL){
		 	 $this->connection = DbConnect::getDefaultConnection();
		 }
		 else{
		 	$this->connection = $dbConnectObj;
		 }
		 	
		 $this->isDevelopmentServer = true;
	}
	
	static function getSingleton(DbConnect $dbConnectObj = NULL){
		
		if(empty(self::$singleton)){
			self::$singleton = new DbCmd($dbConnectObj);
		}
		
		return self::$singleton;
	}
	
	static function escapeSQL($x) {
		return mysql_real_escape_string ( $x );
	}
	
	// Escapes SQL including wildcard characters such as "%" and "_".
	// Passing in underscores to web forms is a good way to hack ;)
	static function escapeLikeQuery($qryStr) {
		
		$qryStr = mysql_real_escape_string ( $qryStr );
		$qryStr = preg_replace("/%/", "\\%", $qryStr);
		$qryStr = preg_replace("/_/", "\\_", $qryStr);
		
		return $qryStr;
	}
	
	static function convertUnixTimeStmpToMysql($unixTimeStamp){
		return date("YmdHis", $unixTimeStamp);
		
	}
	
	// This method will execute any type of a query.
	// For UPDATES and INSERTS it is better to use the respective methods.
	// Insert/Update methods automatically match column "data types" and also escape the values. .
	function query($query) {
		$this->queryStr = $query;
		return $this->fireSQL();
	}
	
	// Field names are the "array keys" and the database values are the "array values".
	// Returns new "Auto Increment ID".
	function insertQuery($tableName, array $fieldsArr) {

		$this->queryStr =& DbHelper::getInsertQueryStr($this->connection->handle, $tableName, $fieldsArr);
		if ($this->fireSQL()){
			return mysql_insert_id ( $this->connection->handle );
		}
			
		// Only return "duplicate key errors" which may be useful in some critical race scenarios.
		if ($this->getLastErrorNum () == 1062){
			return "*DUPKEYERROR*"; 
		}
		else{
			throw new Exception("Error: Insert Query:\n" . $this->getLastErrorDesc () );
		}
	}
	
	// Field names are the "array keys" and the database values are the "array values".
	function updateQuery($tableName, array $fieldsArr, $qualifier) {
		
		$this->queryStr =& DbHelper::getUpdateQueryStr($this->connection->handle, $tableName, $fieldsArr, $qualifier);
		$this->fireSQL ();
	}
	
	private function fireSQL() {
		
		$this->result = mysql_query ( $this->queryStr, $this->connection->handle );
		
		if ($this->result == FALSE){
			$errorDesc = "Error: Query Failed\n";
			
			if($this->isDevelopmentServer){
				$errorDesc .= $this->getLastErrorDesc () . "\n" . $this->queryStr;
			}
			
			throw new Exception($errorDesc);
		}
		return $this->result;
	}
	
	// Get number of rows in current result set.
	function getNumRows() {
		return mysql_num_rows ( $this->result );
	}
	
	// Position to Row in the current result set.
	// RowNum should be between 0 and Total number of rows-1.
	function seekRow($rowNum) {
		
		$rowNum = intval($rowNum);
		
		if (! mysql_data_seek ( $this->result, $rowNum )){
			throw new Exception("Error: Cannot seek to row $rowNum" . mysql_error ());
		}
	}
	
	// Fetch the current row from the result set.
	// Returns null if no more rows available.
	// The row position is incremented following the fetch.
	// Returns an associative array containing field names and values.
	function getRow() {
		return mysql_fetch_assoc ( $this->result );
	}
	
	// Fetch the first value of the result set.
	// The row position is incremented following the fetch.
	// Returns the value from the FIRST COLUMN in each row.
	// If there are no results it returns null.
	function getValue() {
		
		$result = mysql_fetch_array ( $this->result );
		
		if (!isset($result[0])){
			return null; 
		}
		else{
			return $result[0];
		}
	}
	
	// If there are many rows it will return an array of all values in the FIRST column.
	// If no rows exist, this will return an empty array.
	function getValueArr(){
		
		$retArr = array();
		
		while($row = $this->getRow()){
			$retArr[] = current($row);
		}
		
		return $retArr;
	}
	
	function __destruct() {
		if($this->result){
			@mysql_free_result ( $this->result );
		}
	}
	
	// Returns description of last error or empty string "" if no error occurred.
	function getLastErrorDesc() {
		return mysql_error ( $this->connection->handle );
	}
	
	// Returns number of last error or zero if no error occurred.
	function getLastErrorNum() {
		return mysql_errno ( $this->connection->handle );
	}
	
	// Format to DB datetime format
	static function formatDateTime($timestamp) {
		return strftime ( "%Y-%m-%d %H:%M:%S", $timestamp );
	}
}

