<?php


class Inventor {

	private $inventorID;
	private $name;
	private $password;
	private $isTeacher;
	private $birthday;
	private $imageID;
	private $imageDesc;
	private $dbCmd;
	
	
	function __construct(){
		$this->dbCmd = new DbCmd();
	}
	
	function checkIfExists($inventorID){
			
		$this->dbCmd->query("SELECT COUNT(*) FROM inventors WHERE ID=" . intval($inventorID));
		
		if($this->dbCmd->getValue() == "0"){
			return false;
		}
		else{
			return true;
		}
	}
	
	function getImageDesc(){
		
		$this->ensureInventorIsLoaded();

		return $this->imageDesc;
	}
	
	function getImageID(){
		
		$this->ensureInventorIsLoaded();
		
		return $this->imageID;
	}
	
	function updateImageID($imageID){
		
		$this->ensureInventorIsLoaded();
		
		$this->dbCmd->updateQuery("inventors", array("ImageID"=>$imageID), ("ID=" . $this->inventorID));
	}
	
	function updateImageDesc($desc){
		
		$this->ensureInventorIsLoaded();
		
		$this->dbCmd->updateQuery("inventors", array("ImageDescription"=>$desc), ("ID=" . $this->inventorID));
	}
	
	// Pass in TRUE to change the Inventor's "Thought Status" to a TEACHER, otherwise they will become a STUDENT.
	function changeToTeacher(){
		
		$this->ensureInventorIsLoaded();
		
		$this->dbCmd->updateQuery("inventors", array("IsTeacher"=>true), ("ID=" . intval($this->inventorID)));
	}

	function changeToStudent(){
		
		$this->ensureInventorIsLoaded();
		
		$this->dbCmd->updateQuery("inventors", array("IsTeacher"=>false), ("ID=" . intval($this->inventorID)));
	}

	
	function getID(){
		
		$this->ensureInventorIsLoaded();
		return $this->inventorID;
	}
	
	// Make sure to check if the Inventor Name exists before calling this method.
	// The front-end should display a "pretty error" instead of dumping an ugly "exception".
	// This method will return the new InventorID upon success.
	function createInventor($inventorName, $pw){
		
		if(!empty($this->inventorID)){
			throw new Exception("You can't create a new account with an object that has an inventor ID already loaded.");
		}
		
		$inventorName = $this->cleanInventorName($inventorName);
		
		if($this->checkIfNameExists($inventorName)){
			throw new Exception("The Inventor already exists.");
		}
		
		$insertArr["Name"] = $inventorName;
		//$insertArr["IsTeacher"] = false;
		$insertArr["Password"] = $pw;
		$insertArr["ImageID"] = 0;
		$insertArr["ImageDescription"] = "";
		$insertArr["Birthday"] = time();
		
		$this->name = $inventorName;
		$this->birthday = time();
		$this->password = $pw;
		
		$this->inventorID = $this->dbCmd->insertQuery("inventors", $insertArr);
		
		return $this->inventorID;
	}
	
	private function ensureInventorIsLoaded(){
		if(empty($this->inventorID)){
			throw new Exception("The inventor must be loaded before calling this method.");
		}
	}
	
	function getBirthday(){
		$this->ensureInventorIsLoaded();
		return $this->birthday;
	}
	function getPassword(){
		$this->ensureInventorIsLoaded();
		return $this->password;
	}
	function checkIfTeacher(){
		$this->ensureInventorIsLoaded();
		return $this->isTeacher;
	}
	function getName(){
		$this->ensureInventorIsLoaded();
		return $this->name;
	}
	
	// Returns TRUE is the Password matches the entry for the InventorID in the DB, FALSE otherwise.
	// You must load the inventor into this object using another method call.
	function checkInventorCredentials($inventorID, $pw){
		
		$this->dbCmd->query("SELECT Password FROM inventors WHERE ID=". intval($inventorID));
		
		if($this->dbCmd->getNumRows() != 1){
			throw new Exception("The Inventor ID does not exist.");
		}
	
		$passwordFromDb = $this->dbCmd->getValue();
		
		if($passwordFromDb === null){
			$passwordFromDb = "";
		}
		
		if($passwordFromDb === $pw){
			return true;
		}
		else{
			return false;
		}
	}
	
	function getInventorID($inventorName){
		
		$inventorName = $this->cleanInventorName($inventorName);
		
		$this->dbCmd->query("SELECT ID FROM inventors WHERE Name LIKE '". DbCmd::escapeLikeQuery($inventorName) ."'");
		
		if($this->dbCmd->getNumRows() != 1){
			throw new Exception("The Inventor Name does not exist.");
		}
		
		return $this->dbCmd->getValue();
	}
	
	function loadInventor($id){
		
		if(!empty($this->inventorID)){
			throw new Exception("You can't load another inventor. Create another object instead.");
		}
		
		$this->dbCmd->query("SELECT *, UNIX_TIMESTAMP(Birthday) AS Birthday FROM inventors WHERE ID=" . intval($id));
		if($this->dbCmd->getNumRows() != 1){
			throw new Exception("The Inventor ID does not exist.");
		}
		
		$row = $this->dbCmd->getRow();
		
		$this->name = $row["Name"];
		$this->isTeacher = empty($row["IsTeacher"]) ? false : true;
		$this->birthday = $row["Birthday"];
		$this->imageID = $row["ImageID"];
		$this->imageDesc = $row["ImageDescription"];
		$this->password = $row["Password"];
		
		$this->inventorID = $id;
	}
	
	function checkIfNameExists($inventorName){
		
		$inventorName = $this->cleanInventorName($inventorName);
		$this->dbCmd->query("SELECT COUNT(*) FROM inventors WHERE Name LIKE '". DbCmd::escapeLikeQuery($inventorName) ."'");
		if($this->dbCmd->getValue() > 0){
			return true;
		}
		else{
			return false;
		}
	}
	
	function updatePassword($pw){
		
		$this->ensureInventorIsLoaded();
		
		$this->dbCmd->updateQuery("inventors", array("Password"=>$pw), ("ID=" . $this->inventorID));
	}
	
	// Cleans the "Inventor Name"
	// Make sure people aren't trying to hack, or accidently inject newline characters, tabs, etc.
	private function cleanInventorName($inventorName){
		
		$inventorName = trim($inventorName);
		$inventorName = filter_var($inventorName, FILTER_SANITIZE_STRING, FILTER_FLAG_STRIP_LOW | FILTER_FLAG_NO_ENCODE_QUOTES);
		
		if(empty($inventorName)){
			throw new Exception("The Inventor Name can not be blank.");
		}
		return $inventorName;
	}
	
	// Pass in a string to use as a prefix for creating the guest account.
	// Method will return the new Inventor ID.
	function createGuestAccount($prefix){
		
		$newInventorName = $this->cleanInventorName($prefix);
		
		// Our DB column is currently limited to 255 chars.
		// Make sure we won't exceed that with a new suffix.
		if(strlen($newInventorName) > 220){
			$newInventorName = substr($newInventorName, 0, 220);
		}
		
		$newInventorName .= "_Guest_";
		
		// Let's use the date/time as the suffix to decrease our chances of collissions.
		$counter = 0;
		while(true){
			
			$suffix = microtime();
			
			if(!$this->checkIfNameExists($newInventorName . $suffix)){
				$newInventorName = $newInventorName . $suffix;
				break;
			}
			
			$counter++;
			
			if($counter > 1000){
				throw new Exception("Infinite loop detected in method createGuestAccount");
			}
		}
		
		return $this->createInventor($newInventorName, "");	
	}
	
}

