<?php

 
// This class contains static methods to reduce the weight of each Neuron Object.
// There could be a staggering number of neuron objects created recursively.  
// The var-dumps show "methods" and "variables" carried with each instance.
// Perhaps the engine is smart enough to avoid redundant copies, however the var-dumps still display. 
// The more code that we can move into here, the better.
class NeuronHelper {
	
	public static $queries_L1 = array();
	public static $inserts_L1 = array();
	public static $queries_C27_Top = array();
	public static $queries_C27_Bottom = array();
	
	static function getCharacterStreamFromContextArr(array &$contextArr){
		
		$retArr = array();
		
		foreach($contextArr as $loop_ContextItem){
			
			$charsArr = self::getCharacterArrFromString($loop_ContextItem);
			
			foreach($charsArr as $loop_Char){
				$retArr[] = $loop_Char;
			}
			
			$retArr[] = SYMBOL_END;
		}
		
		return $retArr;
	}
	
	static function getCharacterArrFromString($data){
		return preg_split('//', $data, -1, PREG_SPLIT_NO_EMPTY);
	}
	
	// Whenever you are entering data, it must be entered within context of Odd/Even.
	// The person who starts the conversation never ends it.
	// If the conversation is polite, it will end with Flip=Thanks, Flop=Your+Welcome
	static function getPrimaryDataSignature($inventorID, $flipFlopFlag){
		
		$inventorID = intval($inventorID);
		
		$inventorObj = new Inventor();
		if(!$inventorObj->checkIfExists($inventorID)){
			throw new Exception("The Inventor ID is invalid.");
		}
		
		if(!is_bool($flipFlopFlag)){
			throw new Exception("The flip-flop flag must be Boolean.");
		}
		
		if($flipFlopFlag){
			$inventorDesc = $inventorID . "-flip";
		}
		else{
			$inventorDesc = $inventorID . "-flop";
		}
		
		return md5("The primary/root data is entered within the context of an Inventor ID: " . $inventorDesc);
	}
	
	static function doL1Query(array &$contextArr, &$childSignature, &$selfCharacterCode){

		if(empty($contextArr)){
			throw new Exception("You can't do an L1 Query without anything to query on.");
		}
		
		// For reporting
		if(!isset(self::$queries_L1[$selfCharacterCode])){
			self::$queries_L1[$selfCharacterCode] = 1;
			self::$inserts_L1[$selfCharacterCode] = 0;
		}
		else{
			self::$queries_L1[$selfCharacterCode]++;
		}
		
		$triangleObj =& Triangle::getSingleton();
		$dbCmd =& DbCmd::getSingleton();
		
		$L1_ColumnQuery = $triangleObj->getL1($contextArr);
		
// I just had doubts over whether we should be querying on "L1" verses "Signature" 
		$dbCmd->query("SELECT COUNT(*) FROM t".DbCmd::escapeLikeQuery($selfCharacterCode)."
							WHERE ChildSignature ='".DbCmd::escapeSQL($childSignature)."' 
							AND Signature = '".DbCmd::escapeSQL(self::getContextSignature($contextArr))."'");
							//AND " . DbCmd::escapeLikeQuery(key($L1_ColumnQuery)) . " = '".DbCmd::escapeSQL(current($L1_ColumnQuery))."'");
		
		// If we found results, then it means the pattern is familiar, so we don't need to write anything to the DB.
		if($dbCmd->getValue() > 0){
			return;
		}
		
		// For reporting
		self::$inserts_L1[$selfCharacterCode]++;

		// Build up to 27 Context Hashes for the row we are about to insert.
		// If the context array holds less than 6 items, this method will not return all 27 columns.
		// This will get our "insert hash" started.  We still have 3 more columns to add before insertion.
		$insert = $triangleObj->getC27($contextArr);
		
		// Every time this Female inserts records data, it must be stored in context of its decendent's own "context signature".
		$insert["ChildSignature"] = $childSignature;
		
		// This is a digestion of the last 6 items we have received in our "Context Array".
		// For FamilyTree objects closer to the root, each of the 6 context items may be comprised of long strings which aggregate output from their ancestors.
		// No single FamilyTree object can remember more than 6 things.  As knowledge is increased, so is the sophistication of the 6 things which can be remembered.
		$insert["Signature"] = $triangleObj->getSignatureFromContextChain($contextArr); 
		
		// The date is important because we want to return the newest data (if there are multiple matching signatures in context).
		// When people begin sharing their knowledge through replication, we won't be able to use an Auto-Increment ID for this purpose.
		$insert["Date"] = time(); 
		
		$dbCmd->insertQuery("t".$selfCharacterCode, $insert);
	} 
	
	static function getContextSignature(array &$contextArr){

		$triangleObj = Triangle::getSingleton();
		
		return $triangleObj->getSignatureFromContextChain($contextArr);
	}
	
	static function getSignatureFromC27Query(array &$contextArr, &$childSignature, &$selfCharacterCode){

		$dbCmd =& DbCmd::getSingleton();
		$triangleObj =& Triangle::getSingleton();
		
		// For reporting
		if(!isset(self::$queries_C27_Top[$selfCharacterCode])){
			self::$queries_C27_Top[$selfCharacterCode] = 1;
			self::$queries_C27_Bottom[$selfCharacterCode] = 0;
		}
		else{
			self::$queries_C27_Top[$selfCharacterCode]++;
		}
		
		$c27Hash = $triangleObj->getC27($contextArr);
	
		$retValue = null;
		
		foreach($c27Hash as $loop_Column => $loop_Signature){
			
			// For reporting
			self::$queries_C27_Bottom[$selfCharacterCode]++;
			
			// Order by Date DESC to find the most recent match (if there are multiple).
			$dbCmd->query("SELECT Signature FROM t".DbCmd::escapeLikeQuery($selfCharacterCode)." 
									WHERE ChildSignature ='".DbCmd::escapeSQL($childSignature)."' 
									AND " . DbCmd::escapeSQL($loop_Column) . " = '". DbCmd::escapeSQL($loop_Signature) . "' 
									ORDER BY DATE DESC LIMIT 1");
			$retValue = $dbCmd->getValue();
			
			if($retValue){
				break;
			}
		}
		
		return $retValue;
	}
}


