<?php
 

// OUTPUT Neuron
class Boy extends Neuron {
	
	function __construct(FamilyTree $myFamilyObj){
		parent::__construct($myFamilyObj);
	}
	
	// Once our father has finished sending us data, we can just relay the information directly to our child.
	// If our child happens to be a son, he can do the same exact thing to his child.
	function finishedReceivingFathersAdvice(){
		
		//$this->myFamilyObj->hasNonEmptyCharacterGoneToParent = false;
		
		// Between generations we need to dynamically and link child objects to this Neuron.
		$this->linkNewGenerationChildObjects();

		//print Structure::getSpacesFromCharacterCode($this->myFamilyObj->selfCharacterCode) . "<font color=blue>Boy: (".$this->myFamilyObj->selfCharacterCode.") Finished Receiving Father's advice</font><br>";
		// The parent method didn't add the "End Symbol".
		// We have to do that now so that our "Context Array" can be incremented.
		$this->myFamilyObj->receiveCharacter(SYMBOL_END);

		$this->myFamilyObj->setCharacterStreamFromContextArr();
		
		// Now that the stream has been set in memory, we can clear out the "Context Array" which may get re-filled soon by our wife or father.
		$this->myFamilyObj->resetContext();
		
		foreach($this->myFamilyObj->characterStreamArr as $loop_Char){
		
			// Don't let our father relay "END Symbols" that were indended for our current level.
			if($loop_Char == SYMBOL_END){
				continue;
			}
			
			$this->outputCharacter($loop_Char);
		}
		
		// To signal that we are done sending the data stream.
		$this->outputCharacter(SYMBOL_END);
	}

	// This method will only be called by our wife, after she received an "END Symbol".
	function honeyDo(){
//print Structure::getSpacesFromCharacterCode($this->myFamilyObj->selfCharacterCode) . "<b>Boy</b>: (".$this->myFamilyObj->selfCharacterCode.")" . $this->myFamilyObj->relationship . "  HoneyDo<br>";	
		
		// Between generations we need to dynamically and link child objects to this Neuron.
		$this->linkNewGenerationChildObjects();
		

		// Our wife is supposed to forward the "End Symbol" (with an empty "Context Array") to her mother.
		// However, after that "Parent Chain" has finished its thread, the program execution will return here.
		// It is possible that the wife could have started off with an empty array and it was filled up (by the ancestors) before getting here, or vica versa.
		if(empty($this->myFamilyObj->contextArr)){
//print "Empty Context for Boy<br>";	
			$this->outputCharacter(SYMBOL_END);
			return;
		}
//print "About to do C27<br>";
//var_dump($this->myFamilyObj->contextArr);
//print "<br><br>";


		// Our wife just ensures that nothing is unfamiliar in context of the "Child Signature" and the last "Context Item" (L1 Query).
		// L1 is puny, there are 26 other scenarios which are more favorable (depending on how many "Context Items" we have).  
		// We want to completely replace "Context Array" that both of us share before she receives any more characters.
		$this->myFamilyObj->setCharacterStreamFromContextArr();
//var_dump($this->myFamilyObj->contextArr);		
		$c27ContextSignature = NeuronHelper::getSignatureFromC27Query(
														$this->myFamilyObj->contextArr, 
														$this->myFamilyObj->childSignature, 
														$this->myFamilyObj->selfCharacterCode);
		if(empty($c27ContextSignature)){
			$exceptionDetails = "Self Character Code: (" . $this->myFamilyObj->selfCharacterCode . ") Context Array: " . print_r($this->myFamilyObj->contextArr, true);
			throw new Exception("No match on the C27 Query. Details: $exceptionDetails");
		}
		
		// We are going to rebuild our "Context Array" by sending data to our parents (using a different "Child Signature" for them).
		$this->myFamilyObj->resetContext();
		
		foreach($this->myFamilyObj->characterStreamArr as $loop_Char){
			
			// Notice 4 Conditionals (just like the bases of Egyptian pyramids).  The "Great Pyramid" was missing the capstone too.
			// Scientists have noticed that the 4th layer in the Neocortex is star shaped (with 4 points) unlike the rest of the "pyramidal cells".
			if(empty($this->parentsObj)){
				
				// We are a Caveman.  There is no other place to send the data to, other than the top of the pyramid.
				$this->outputCharacter($loop_Char);
			}
			else if($loop_Char == SYMBOL_END){
	
				// This method will use the "END Symbol" to create new entries on the "Context Array".
				// The "End Symbols" are for us (at this level).  When we stream the data back to our child, we will only send a single "End Symbol" when we are done.
				$this->myFamilyObj->receiveCharacter(SYMBOL_END);
			}
			else if(ord($loop_Char) == $this->myFamilyObj->selfCharacterCode){
				
				// This will prompt our parents to return any data which they may be holding within their buffer.
				// We want to get that stuff back before adding our "current character code" onto the array.
				if($this->myFamilyObj->hasNonEmptyCharacterGoneToParent){
					$this->parentsObj->girlNeuron->inputData($c27ContextSignature, SYMBOL_END);
				}

				$this->myFamilyObj->receiveCharacter($loop_Char);
			}
			else{

				// Since we have parents with language ability, pass down the character to make use of their wisdom.
				// We know that we are not a caveman because any character which makes it to the bottom would have matched the "Self Character Code".
				$this->myFamilyObj->hasNonEmptyCharacterGoneToParent = true;
				$this->parentsObj->girlNeuron->inputData($c27ContextSignature, $loop_Char);	
			}
		}
		
		// If we have been sending data to our parents, let them know that the stream is done.
		// They will call the method (on our current level) for finishedReceivingFathersAdvice.
		// Indirectly we will outputting an "End Symbol" to our child if we send an "End Symbol" to our parents.
		if($this->parentsObj){
			if($this->myFamilyObj->hasNonEmptyCharacterGoneToParent){
				$this->parentsObj->girlNeuron->inputData($c27ContextSignature, SYMBOL_END);	
			}
		}
		else{
//print "<u>Caveboy About to Output</u><br>";			
			// Cavemen don't get notified with an END Symbol.
			// We have to close off the "Context Array" manually before we can build the character stream.
			$this->myFamilyObj->receiveCharacter(SYMBOL_END);
			$this->myFamilyObj->setCharacterStreamFromContextArr();
			$this->myFamilyObj->resetContext();
		
			// Since we don't have any parents, there is only one place left to go.  
			// What goes down, must come up.
			foreach($this->myFamilyObj->characterStreamArr as $loop_Char){
				$this->outputCharacter($loop_Char);
			}

			$this->outputCharacter(SYMBOL_END);
		}
	}
	
	// It is possible that we don't have a child.
	// If we are on the "Root Family Tree" object, just rebuild our own "Context Array" and let that class dig out the "return data" manually.
	private function outputCharacter($character){
//print "<font color='maroon'><b>Boy Output</b>:" .  htmlspecialchars($character) . "</font><br>";			
		if($this->childObj){
//print "<font color=blue>Has Child</font><br>";
			// Polymorphism: This method exists for both sons and daughters.
			$this->childObj->fathersAdvice($character);
		}
		else{
//print "<font color=red>Child-less</font><br>";
			$this->myFamilyObj->receiveCharacter($character);
		}
	}
}


