<?php
/*
	\\-------------------------------------------------------//
	/* $100 * * * * * * * * * * * * * * * * * * * * *  $100 *\
	|*                                                      *|
	|*                                                      *|
	|*      It's family that makes the world go round.      *|
	|*                 - In God We Trust -                  *|
	|*                                                      *|
	\* $100 * * * * * * * * * * * * * * * * * * * * *  $100 */
	//-------------------------------------------------------\\
	

define("RELATIONSHIP_SELF", "1");
define("RELATIONSHIP_PARENTS", "2");
define("RELATIONSHIP_GRANDPARENTS", "3");

class FamilyTree {

	public static $structureObj;
	
	public $decendentCharacterCode;
	
	public $boyNeuron;
	public $girlNeuron;

	public $isCaveCouple;
	
	public $characterBuffer;
	public $contextArr = array();
	public $contextSignature;
	
	public $childSignature;
	
	public $selfCharacterCode;
	
	public $characterStreamArr = array();
	
	public static $familyTracker = array();
	
	public $relationship;
	
	static $objectCollection = array();
	
	public $hasNonEmptyCharacterGoneToParent;
	
	private function __construct($descedentCharacterCode = null){

		if(empty(self::$structureObj)){
			self::$structureObj =& Structure::getSingleton();
		}
		
		if(self::$structureObj->getTotalCharacterCount() % 3 != 0){
			throw new Exception("The total character codes in the cardinality set must be divisible by 3.");
		}
		
// Not sure if this is working or not... have another problem at the moment. 
$this->hasNonEmptyCharacterGoneToParent = true;
		
		$this->decendentCharacterCode = $descedentCharacterCode;
		
		$this->characterBuffer = "";
		
		// In a Chat application, the Keyboard and Monitor are connected the Family Tree object which does not have any decendents.
		// It is the Root input/output.
		if(empty($this->decendentCharacterCode)){
			$this->selfCharacterCode = self::$structureObj->getCharacterCodeWithLowestCardinality();
			
			$this->relationship = RELATIONSHIP_SELF;
		}
		else{
			// If we have a decendent we can figure out what their character code is.
			// Then we can figure out our character code on the current level based upon the cardinality chain from the training set.
			$this->selfCharacterCode = self::$structureObj->getCharacterCodeWithNextHighestCardinality($this->decendentCharacterCode);
			
			// Every third character code the relationship goes back to "self".
			$this->relationship = self::$structureObj->getRelationshipInCardinalityChain($this->selfCharacterCode);
		}

		// So that we can keep track of how many objects are being created.
		if(!isset(self::$familyTracker[$this->selfCharacterCode])){
			self::$familyTracker[$this->selfCharacterCode] = 1;
		}
		else{
			self::$familyTracker[$this->selfCharacterCode]++;
		}
/*
var_dump(self::$familyTracker);
print "<br><br>";
usleep(900);
flush();
*/
		// The following 2 neurons will not have their "Child Objects" set.
		// They are the new generation, which means that they should be the wisest.
		$this->boyNeuron =& new Boy($this);
		$this->girlNeuron =& new Girl($this);
		
		// We know if the Neuron is a caveman if they don't have any knowledge from their ancestors to pass down.
		// That means, they sensed the most rudimentary pattern.  
		// From a coding perspective, we can't find a character code with a higher cardinality value.
		// We have to look ahead in the cardinality chain to know if we are a caveman or not.
		$ancestorCharacterCode = self::$structureObj->getCharacterCodeWithNextHighestCardinality($this->selfCharacterCode);
		
		$this->isCaveCouple = empty($ancestorCharacterCode) ? true : false;
		
		// If this branch on the family tree is not a caveman, then we need to need to recursively create another "Family Tree" object.
		// Remember that we are going in reverse.  We are not creating children... we are creating ancestors!
		// The sun is moving, not us. Just look at it!
		if(!$this->isCaveCouple){
	
			// As long as we are not "Grandparents", then it is safe to create a new FamilyTree object in memory.
			// Otherwise, we want to use a pointer to a new/single generation (which will contain at most 3 levels).
			if($this->relationship != RELATIONSHIP_GRANDPARENTS){
				$this->boyNeuron->parentsObj =& new FamilyTree($this->selfCharacterCode);
				$this->girlNeuron->parentsObj =& new FamilyTree($this->selfCharacterCode);
				
				// Create references for each of the parents so they know who their 1 and only child is.
				// Use polymorphism with the child objects.
				$this->boyNeuron->parentsObj->boyNeuron->childObj =& $this->boyNeuron;
				$this->boyNeuron->parentsObj->girlNeuron->childObj =& $this->boyNeuron;
	
				$this->girlNeuron->parentsObj->boyNeuron->childObj =& $this->girlNeuron;
				$this->girlNeuron->parentsObj->girlNeuron->childObj =& $this->girlNeuron;
			}
			else{
				$this->boyNeuron->parentsObj =& self::getNewGeneration($this->selfCharacterCode);
				$this->girlNeuron->parentsObj =& self::getNewGeneration($this->selfCharacterCode);
				
				// For GrandParents, we have to Switch the Child Object "on the fly" depending on whether the Boy or the Girl calls the next generation.
			}
			

		}
	}
	
	// This is not a singleton, however the constructor has been made private (for similar reasons).
	// Instead of exponentially growing our pyramid to an incomprehensionable, we can share objects which have already been created.
	static function getNewGeneration($descedentCharacterCode = null){
		
		if(empty(self::$structureObj)){
			self::$structureObj =& Structure::getSingleton();
		}
		
		// If there are no Descedents, then we may have to create a new FamilyTree object based upon the root character code.
		if(empty($descedentCharacterCode)){
			
			$rootCharacterCode = self::$structureObj->getCharacterCodeWithLowestCardinality();

			// Add it to our Static Array before returning the reference.
			if(!array_key_exists($rootCharacterCode, self::$objectCollection)){
				
				// Prevent Recurrsion
				self::$objectCollection[$rootCharacterCode] = null;
				self::$objectCollection[$rootCharacterCode] =& new FamilyTree();
			}

			return self::$objectCollection[$rootCharacterCode];
		}
		
		// We can determine the new character code based upon the character code of our child.
		// Then we will see if an object has already been created or not.
		$newCharacterCode = self::$structureObj->getCharacterCodeWithNextHighestCardinality($descedentCharacterCode);
		
		if(!array_key_exists($newCharacterCode, self::$objectCollection)){
			
			// Prevent Recurrsion
			self::$objectCollection[$newCharacterCode] = null;
			self::$objectCollection[$newCharacterCode] =& new FamilyTree($descedentCharacterCode);
		}
		
		return self::$objectCollection[$newCharacterCode];
	}
	
	// This will set an array of characters which get extracted from "Context Array".
	// Between each "Context Item", there will be a division of BEGIN/END Symbols.
	function setCharacterStreamFromContextArr(){
		$this->characterStreamArr = NeuronHelper::getCharacterStreamFromContextArr($this->contextArr);
	}
	
	function resetContext(){
		$this->contextArr = array();
		$this->contextSignature = md5(SYMBOL_END);
	}
	
	// Both Input & Output Neurons receive characters.
	function receiveCharacter($character){
		
		$recalculateSignature = false;
		
		// The Context Array is reset when the child signature changes.
		// When we receive and "End Signal" we can append our buffer to the Context Array.
		if($character == SYMBOL_END){
			
			// If we just get a BEGIN/END Symbol with nothing in between, don't add anything to our "Context Array".
			if(!empty($this->characterBuffer)){
				$this->contextArr[] = $this->characterBuffer;
				$recalculateSignature = true;
			}
			
			// It is only possible to hold 6 things within our short term memory.
			if(sizeof($this->contextArr) > CONTEXT_LENGTH){
				array_shift($this->contextArr);
			}
			
			// Clear it out for the next round.
			$this->characterBuffer = "";
		}
		else{
			
			// Until we get the "END Signal", just keep filling up our buffer.
			$this->characterBuffer .= $character;
		}
		
		// Reducing the number of times that we calculate the signatures for the Girl Neurons will increase performance a little.
		if($recalculateSignature){
			$this->contextSignature = NeuronHelper::getContextSignature($this->contextArr);
		}
	}
	
	// Whenever you are entering data, it must be entered within context of Odd/Even.
	// The person who starts the conversation never ends it.
	// If the conversation is polite, it will end with Flip=Thanks, Flop=Your+Welcome
	// This method returns the "output".
	function inputRoot($data, $inventorID, $flipFlopFlag){
		
		$this->validateRoot();
		
		if(!is_bool($flipFlopFlag)){
			throw new Exception("The flip-flop flag must be Boolean.");
		}
		
		$data = self::$structureObj->getCleanData($data);
		
		$childSignature = NeuronHelper::getPrimaryDataSignature($inventorID, $flipFlopFlag);
		
		// The child structure has a different clean-up routine that the adult stucture.  One may be voice, the other could be text (or vica versa).
		$charsArr = NeuronHelper::getCharacterArrFromString($data);
		
		// The data must be streamed down the pyramid, one character at at time.
		// The Pyramid Objects may send data to a c
		foreach($charsArr as $loop_Char){
			$this->girlNeuron->inputData($childSignature, $loop_Char);
		}

		// As soon as the "End Symbol" is received, the final level will return our data.
		$this->girlNeuron->inputData($childSignature, SYMBOL_END);
		
		// We manually calling the inputs/outputs within the Root Family Tree object.
		// Our husband is the last Neuron to receive data.
		// It is important we don't forget the roll of the "Girl Neuron".
		// We will call the "output method" shortly, which will require a C27 query (before the changing the "FlipFlop Bit").
		$this->girlNeuron->doL1();
		
		// We don't need to access the "Boy Neuron".  He has already filled up the "Context Array" which the Girl Neuron shares.
		$this->setCharacterStreamFromContextArr();
		
		$this->resetContext();

		// The last "Child Neuron" in the family tree would have already called $this->setCharacterStreamFromContextArr() for us.		
		$retStr = implode("", $this->characterStreamArr);
	
		// There could be some "End Symbols" in the "Character Stream", which we don't want to return.
		$retStr = preg_replace('/'.preg_quote(SYMBOL_END, '/').'/', "", $retStr);
		
		return $retStr;
	}

	
	private function validateRoot(){
		if($this->decendentCharacterCode){
			throw new Exception("The input/output methods may only be called from the initial FamilyTree object.");
		}
	}
}


