<?php



// INPUT Neuron
class Girl extends Neuron {
	
	private $hasNonEmptyCharacterGoneToParent;

	function __construct(FamilyTree $myFamilyObj){
		parent::__construct($myFamilyObj);
		
		//$this->myFamilyObj->hasNonEmptyCharacterGoneToParent = false;
	}

	// Once our father has finished sending us data, we need to do an L1 Query.
	// If there is anything on our "Context Array", our husband will subsequently perform a C27 query.
	function finishedReceivingFathersAdvice(){
		
		print Structure::getSpacesFromCharacterCode($this->myFamilyObj->selfCharacterCode) . "<font color=green>Girl: (".$this->myFamilyObj->selfCharacterCode.") Finished Receiving Father's advice</font><br>";

		// After the father sends us advice, there won't be any data left down in the buffers (with our parents).
		//$this->myFamilyObj->hasNonEmptyCharacterGoneToParent = false;
		
		// Between generations we need to dynamically and link child objects to this Neuron.
		$this->linkNewGenerationChildObjects();
		
		// Remember that Girl Neurons forward data to their parents when they have an empty "Context Array" and they come across an "End Symbol".
		// We don't want to get caught in an infinite loop here.
		if(!empty($this->myFamilyObj->contextArr) || !empty($this->myFamilyObj->characterBuffer)){
			$this->inputData($this->myFamilyObj->childSignature, SYMBOL_END);
		}
	}

	// This method will only be called by Sons and Daughters (except for the "Family Tree Root").
	function inputData($childSignature, $character){

print Structure::getSpacesFromCharacterCode($this->myFamilyObj->selfCharacterCode) . "Girl: (".$this->myFamilyObj->selfCharacterCode.")" . $this->myFamilyObj->relationship . "  InputData($character)<br>";
usleep(2900); flush();

		// Between generations we need to dynamically and link child objects to this Neuron.
		$this->linkNewGenerationChildObjects();
		
		
		$this->validateInput($childSignature, $character);
		
		// When the child signature changes, we have a completely new context to start interpretting our character stream with.
		if($childSignature != $this->myFamilyObj->childSignature){
			$this->myFamilyObj->resetContext();
		}
		
		$this->myFamilyObj->childSignature = $childSignature;

		// Notice 4 Conditionals (just like the bases of Egyptian pyramids).  The "Great Pyramid" was missing the capstone too.
		// Scientists have noticed that the 4th layer in the Neocortex is star shaped (with 4 points) unlike the rest of the "pyramidal cells".
		if(empty($this->parentsObj)){
//print "<font color='orange'>CaveGirl:" .  ord($character) . "</font><br>";		
			
			if(ord($character) != $this->myFamilyObj->selfCharacterCode && $character != SYMBOL_END){
				throw new Exception("At the bottom of the cardinality chain (where there are no parents), the character code should match.");
			}
			
			// Since we are a Cavewoman, every character that we receive will be added as a separate element on the "Context Array".
			if($character != SYMBOL_END){
				$this->myFamilyObj->receiveCharacter($character);
				$this->myFamilyObj->receiveCharacter(SYMBOL_END);
			}
			else{
				$this->myFamilyObj->receiveCharacter(SYMBOL_END);
			}

			// The L1 here is not going to be complicated.  The only choices for a "Previous Context Item" (for a Cavewoman) are "empty context" or its own character code.
			// Of course, there could be LOTS of possibilites written to the DB, depending on the "Child Signature".
			$this->doL1();
			
			$this->myFamilyObj->boyNeuron->honeyDo();
		
		}
		else if($character == SYMBOL_END){

			// END symbols are not supposed to be relayed down the family tree.
			// They are only meant to control the buffer & "Context Array" on our current level.
			// If we don't call this method in the proper sequence, we won't have a new item on our "Context Array" for the "L1 Query".
			$this->myFamilyObj->receiveCharacter(SYMBOL_END);
			
			// This "Neuron Couple" may just be a bridge between the "input source" and an ancestor which has data in its buffer.
			// Remember, the "End Symbol" is only for our current level.  If we are sending down any other data (besides an "End Symbol") the data will naturally return.
			// If we can't send our ancestors any data, then we have to send them an "End Symbol".
			if(empty($this->myFamilyObj->contextArr)){
//var_dump($this->parentsObj->girlNeuron);
//print "Empty Context Array " . $this->myFamilyObj->selfCharacterCode . "-<br>";

				// For performance, don't send down an "End Symbol" until at least 1 character has been send down in the past.
				// If no characters have gone to the parent yet, there won't be anything hiding in the buffers down there.
				if($this->myFamilyObj->hasNonEmptyCharacterGoneToParent){
					$this->parentsObj->girlNeuron->inputData($this->myFamilyObj->contextSignature, SYMBOL_END);	
				}

			}
		
			// We can't do an "L1 Query" if we have an empty "Context Array".
			// It is possible that sending the "End Symbol" to our parent (in the step before) could have filled up our "Context Array" because it may have set off a new chain reaction.
			$this->doL1();

			// As a couple, we share the same "Context Array".  
			// Our husband is just waiting for an "END" signal to let him know that we are done with the "L1 Query" (possibly connecting synapses).
			$this->myFamilyObj->boyNeuron->honeyDo();
		}
		else if(ord($character) == $this->myFamilyObj->selfCharacterCode){
//print "<font color='Red'>Disolving:" .  ord($character) . "</font><br>";
			// This will prompt our parents to return any data which they may be holding within their buffer.
			if($this->myFamilyObj->hasNonEmptyCharacterGoneToParent){
				$this->parentsObj->girlNeuron->inputData($this->myFamilyObj->contextSignature, SYMBOL_END);
			}

			// Append the our "Current Character Code" onto our context buffer after the parents have possibly returned data in their buffer.
			$this->myFamilyObj->receiveCharacter($character);
		}
		else{
			
			$this->myFamilyObj->hasNonEmptyCharacterGoneToParent = true;
			
			// Since we have parents with language abilities, let them handle the "more frequent" patterns.
			$this->parentsObj->girlNeuron->inputData($this->myFamilyObj->contextSignature, $character);	
		}
	}
	
	// Each time that a Girl receives something new on the context array, we need to make sure that the pattern is familiar relative to...
	// a) The "Child Signature" who is sending us data (within their context).
	// b) The "Context Item" which came immediately before (in this level).  The "L1 Query" only looks backwards 1 step in the "Context Array".
	function doL1(){
		
		if(empty($this->myFamilyObj->contextArr)){
			return;
		}
		
		// We don't need to return anything. Just make sure that no pattern is unfamiliar within the database.
		// Our husband's subsequent C27 query may need this now (or in the future).
		NeuronHelper::doL1Query(	$this->myFamilyObj->contextArr, 
									$this->myFamilyObj->childSignature, 
									$this->myFamilyObj->selfCharacterCode);
	}
	
	private function validateInput($childSignature, $character){
		
		if(strlen($childSignature) != 32){
			throw new Exception("The child signature must be digested before calling thid method.");
		}
		
		FamilyTree::$structureObj->validateCharacter($character);
	}
}
