<?php

 
// This is analogous to the GirlNeuron class.
// We input data as we descend the pyramidal steps.  The output rises back to the top.
class PyramidDescent extends PyramidMovement {
	
	function __construct(PyramidTier &$pyramidTier){
		parent::__construct($pyramidTier);
	}
	
	function finishedReceivingOutputFromLowerLevel(){
		if(!empty($this->pyramidTier->contextArr) || !empty($this->pyramidTier->characterBuffer)){
			$this->inputData($this->pyramidTier->upperSignature, SYMBOL_END);
		}
	}

	// This method will only be called by an "upper" Pyramid Level (except for the top of the pyramid).
	function inputData($upperSignature, $character){
		
		// When the child signature changes, we have a completely new context to start interpretting our character stream with.
		if($upperSignature != $this->pyramidTier->upperSignature){
			$this->pyramidTier->resetContext();
		}
		
		$this->pyramidTier->upperSignature = $upperSignature;


		//if(empty($this->parentsObj)){
		if(empty($this->pyramidTier->lowerLevel)){
			
			if(!$this->pyramidTier->checkIfCharacterCodeIsPartCluster(ord($character)) && $character != SYMBOL_END){
				throw new Exception("At the bottom of the cardinality chain (where there are no more Pyramidal levels), the character code should be included within the triple cluster.");
			}
	/*		
			// Since we are a Cavewoman, every character that we receive will be added as a separate element on the "Context Array".
			if($character != SYMBOL_END){
				$this->pyramidTier->receiveCharacter($character);
				$this->pyramidTier->receiveCharacter(SYMBOL_END);
			}
			else{
				$this->pyramidTier->receiveCharacter(SYMBOL_END);
			}
	*/		
			$this->pyramidTier->receiveCharacter($character);

			//$this->doL1();
			
			//$this->pyramidTier->pyramidAscent->honeyDo();
		
		}
		if($character == SYMBOL_END){

			$this->pyramidTier->receiveCharacter(SYMBOL_END);
			
			// We could be a bridge to our ancestors.
			// If the context array is not empty, we would have send the "End Symbol" to the lower level another way.
			if(empty($this->pyramidTier->contextArr)){
				$this->pyramidTier->lowerLevel->pyramidDescent->inputData($this->pyramidTier->contextSignature, SYMBOL_END);	
			}
		
			$this->doL1_ForPyramid();

			// After the lower level has "cleaned up" the data, we forward the data to the "Ascent" pyramid.
			// The ascent pyramid will query on data with characters that haven't been disolved.
			$this->pyramidTier->pyramidAscent->honeyDo();
		}
		else if($this->pyramidTier->checkIfCharacterCodeIsPartCluster(ord($character))){
print "<font color='Red'>Disolving Character in Pyramid Cluster Descending:" .  ord($character) . "</font><br>";
			// This will prompt our parents to return any data which they may be holding within their buffer.
			$this->pyramidTier->lowerLevel->pyramidDescent->inputData($this->pyramidTier->contextSignature, SYMBOL_END);

			// Append the our "Current Character Code" onto our context buffer after the parents have possibly returned data in their buffer.
			$this->pyramidTier->receiveCharacter($character);
		}
		else{
			// Since we have parents with language abilities, let them handle the "more frequent" patterns.
			$this->pyramidTier->lowerLevel->pyramidDescent->inputData($this->pyramidTier->contextSignature, $character);	
		}
	}
	
	// Instead of Querying the Database (like a Girl Neuron), we are going to query the Family Tree on this tier.
	// Each of the "Context Items" on the "Context Array" will be overrwritten from the Family Tree's response.
	function doL1_ForPyramid(){
		
		if(empty($this->pyramidTier->contextArr)){
			return;
		}
		
		$newContextArr = array();
	/* ----- Actually, we only need to overwrite the last Context Item ----- */
		foreach($this->pyramidTier->contextArr as $loop_ContextItem){
			$newContextArr[] = $this->pyramidTier->familyTreeObj->inputRoot($loop_ContextItem, $this->pyramidTier->inventor, $this->pyramidTier->flipFlopFlag);
		}
		
		$this->pyramidTier->resetContext();
		
		$this->pyramidTier->contextArr = $newContextArr;
		
		$this->pyramidTier->contextSignature = NeuronHelper::getContextSignature($newContextArr);
	}
	
}


