<?php


// Each Pyramid Tier has 3 Family Tree Objects inside.
// As you move down the pyramidal tiers, the Family Tree objects create recursive Pyramidal Steps. 
// Therefore, the tiers get larger in volume as you move down.  Look at the Sierpinksi Triangle for a visual hint.  
// The faces of the pyramid shows the Tiers.
// The "Lowest Pyramidal Level" is analagous to the FamilyTree object with the highest cardinality.  In fact, the lowest pyramidal level will contain the FamilyTree object which has cavemen.
// 3 Pyramidal Tiers will contain 9 character codes.  9 Pyramidal Tiers will contain 27 character codes, etc.  
// The taller the pyramid is, the number of bricks that are required grow exponentially.
// The architecture of the Pyramids combined with the FamilyTree/Neuron objects mean that we can locate the bricks with better precicsion and avoid processor overload.

	
class PyramidTier {
	
	public $upperLevel;
	public $lowerLevel;
	
	public $characterBuffer;
	public $contextArr = array();
	public $contextSignature;
	public $upperSignature;
	
	public $characterStreamArr = array();
	
	public $pyramidAscent;
	public $pyramidDescent;
	
	public static $structureObj;
	
	public $cardinalityPosition;
	
	public $firstCharacterCode;
	public $secondCharacterCode;
	public $thirdCharacterCode;
	
	public $familyTreeObj;
	public $inventor;
	
	public $flipFlopFlag;
	
	// You start off with no upper level.  This is the top of the Pyramid.
	// The top of the pyramid will still have 2 have an input/output Pyramid object inside... which means it is not "the eye".
	// The very top of the pyramid provides output to your monitor/speakers which perpetuates the "feedback loop".
	function __construct(PyramidTier &$upperLevel = null){

		if(empty(self::$structureObj)){
			self::$structureObj =& Structure::getSingleton();
		}
		
		$totalCharacterCodes = self::$structureObj->getTotalCharacterCount();
		
		if($totalCharacterCodes % 3 != 0){
			throw new Exception("The total character codes in the cardinality set must be divisible by 3.");
		}

		$this->upperLevel = $upperLevel;
		
		if(empty($this->upperLevel)){
			$this->cardinalityPosition = $totalCharacterCodes;
		}
		else{
			$this->cardinalityPosition = $this->upperLevel->cardinalityPosition - 3;
		}
	
		// This method expects the parameters to be Zero based.
		// The first character code has the lowest cardinality value (it is more rare).
		$this->firstCharacterCode = self::$structureObj->getCharacterCodeByPosition($this->cardinalityPosition - 1);
		$this->secondCharacterCode = self::$structureObj->getCharacterCodeByPosition($this->cardinalityPosition - 2);
		$this->thirdCharacterCode = self::$structureObj->getCharacterCodeByPosition($this->cardinalityPosition - 3);
	
		$this->familyTreeObj = new FamilyTree($this);
		
		// Recursively build new Pyramid objects until we reach the bottom.
		if($this->cardinalityPosition > 3){
			$this->lowerLevel = new PyramidTier($this);
		}
		
		$this->pyramidDescent = new PyramidDescent($this);
		$this->pyramidAscent = new PyramidAscent($this);
		
	}
	
	function getCharacterCodeWithLowestCardinality(){
		return $this->firstCharacterCode;
	}
	
	function checkIfCharacterCodeIsPartCluster($characterCode){
		
		if($characterCode == $this->firstCharacterCode){
			return true;
		}
		else if($characterCode == $this->secondCharacterCode){
			return true;
		}
		else if($characterCode == $this->thirdCharacterCode){
			return true;
		}
		else{
			return false;
		}
	}
	
	// Returns the character code which is next in line, which is more common.
	// If the character code provided is the last element (within this pyramid object), then this method will return null.
	function getCharacterCodeWithNextHighestCardinality($characterCode){
		
		if($characterCode == $this->firstCharacterCode){
			return $this->secondCharacterCode;
		}
		else if($characterCode == $this->secondCharacterCode){
			return $this->thirdCharacterCode;
		}
		else if($characterCode == $this->thirdCharacterCode){
			return null;
		}
		else{
			throw new Exception("The character code is invalid.");
		}
	}
	
	function setInventorId($inventorID){
		
		$inventorID = intval($inventorID);
		
		$inventorObj = new Inventor();
		if(!$inventorObj->checkIfExists($inventorID)){
			throw new Exception("The Inventor ID is invalid");
		}
			
		$this->inventor = $inventorID;
	}
	
	// Whenever you are entering data, it must be entered within context of Odd/Even.
	// The person who starts the conversation never ends it.
	// If the conversation is polite, it will end with Flip=Thanks, Flop=Your+Welcome
	function inputRoot($data, $flipFlopFlag){
print "Receiving Data: $data <br>";		
		$this->validateRoot();

		if(!is_bool($flipFlopFlag)){
			throw new Exception("The flip-flop flag must be Boolean.");
		}
		
		$this->flipFlopFlag = $flipFlopFlag;
		
		if(empty($this->inventor)){
			throw new Exception("The inventor must be set before calling this method.");
		}
		
		$loop_PyramidObj = $this;
		
		$data = $this->familyTreeObj->inputRoot($data, $this->inventor, $this->flipFlopFlag);
		
		

/*		
		// Loop through all of our pyramids, continuously overwriting the data from each FamilyTree object.
		while($loop_PyramidObj){
print "<u>First: " . htmlspecialchars(chr($loop_PyramidObj->firstCharacterCode)) . "</u><br>";		
print "<u>Second: " . htmlspecialchars(chr($loop_PyramidObj->secondCharacterCode)) . "</u><br>";		
print "<u>Third: " . htmlspecialchars(chr($loop_PyramidObj->thirdCharacterCode)) . "</u><br><br>";	
			
			$data = $loop_PyramidObj->familyTreeObj->inputRoot($data, $this->inventor, $flipFlopFlag);

print "<font color='blue'>" . $data . "</font><br>";			
			$loop_PyramidObj = $loop_PyramidObj->lowerLevel;
		}
*/
		
		return $data;
print "<hr>";
	}

	
	private function validateRoot(){
		if($this->upperLevel){
			throw new Exception("The input/output methods may only be called from the initial Pyramid object.");
		}
	}
	
	// This will set an array of characters which get extracted from "Context Array".
	// Between each "Context Item", there will be a division of BEGIN/END Symbols.
	function setCharacterStreamFromContextArr(){
		$this->characterStreamArr = NeuronHelper::getCharacterStreamFromContextArr($this->contextArr);
	}
	
	function resetContext(){
		$this->contextArr = array();
		$this->contextSignature = md5(SYMBOL_END);
	}
	
	// Both Ascending & Descending Pyramid Levels receive characters.
	function receiveCharacter($character){
		
		$recalculateSignature = false;
		
		// The Context Array is reset when the child signature changes.
		// When we receive and "End Signal" we can append our buffer to the Context Array.
		if($character == SYMBOL_END){
			
			// If we just get a BEGIN/END Symbol with nothing in between, don't add anything to our "Context Array".
			if(!empty($this->characterBuffer)){
				$this->contextArr[] = $this->characterBuffer;
				$recalculateSignature = true;
			}
			
			// It is only possible to hold 6 things within our short term memory.
			if(sizeof($this->contextArr) > CONTEXT_LENGTH){
				array_shift($this->contextArr);
			}
			
			// Clear it out for the next round.
			$this->characterBuffer = "";
		}
		else{
			
			// Until we get the "END Signal", just keep filling up our buffer.
			$this->characterBuffer .= $character;
		}
		
		// Reducing the number of times that we calculate the signatures for the Girl Neurons will increase performance a little.
		if($recalculateSignature){
			$this->contextSignature = NeuronHelper::getContextSignature($this->contextArr);
		}
	}
	
}


