<?php


// PHPlib's Class is an excelent and lightweight HTML Template parsing system.
// This just adds a few extra features.
class Templatex extends Template
{

	// Pass in a file system path to load the templates off of ... or a relative path.
	// If you don't specify a path then the Domain Object will try to figure out the HTML Sandbox directory based upon the domain name in the URL.
	function __construct($documentRoot = null){
	
		$this->setDirectoryRoot($documentRoot);
		
		parent::set_unknowns("keep");
	}
	
	function setDirectoryRoot($documentRoot = null){
		
		if(!empty($documentRoot)){
			parent::set_root($documentRoot);
		}
	}

	// Discards block if $condition == true (optional)
	// Benign to call if the block does not exist.
	function discard_block( $parent, $varname, $condition=true )
	{
        if (!parent::loadfile($parent)) {
          parent::halt("set_block: unable to load $parent.");
          return;
        }

		// Make sure that it won't fail in case discard_block is called more than once
		$str = parent::get_var($parent);
		$reg = "/<!--\s+BEGIN $varname\s+-->(.*)\s*<!--\s+END $varname\s+-->/sm";
		if(!preg_match($reg, $str))
	        return;
	
		if( $condition )
		{
			parent::set_block( $parent, $varname, "___discard___" );
			parent::set_var( "___discard___", null ); 
		}
	}
	
	// Copies block to variable names varname. Replaces block in parent with varname,
	// and then parses block in variable varname, if $condition == true
	// If $condition == false, block is discarded from parent
	function parse_block( $parent, $varname, $condition=true )
	{
		if( $condition ){
			parent::set_block( $parent, $varname );
			parent::parse( $varname, $varname );		
		}
		else{
			$this->discard_block( $parent, $varname );	
		}
	}

	function filename($filename){

		// Find out if there is a name/value pair in the URL that specifies a different template number.
		// Normally templates have the prefix "-template.html".  If we have a Template Number in the URL then add that to the prefix.
		// We may overrite the filename with something like "ShoppingCart-template_3.html
		$templateID = WebUtil::GetInput("TemplateID", FILTER_SANITIZE_WORD_OR_NUMBER_NO_SPACES);

		if(preg_match('/template\\.html$/', $filename) && preg_match('/^(\d|\w)+$/', $templateID)){	
			if(strlen($templateID) > 25)
				exit("The Template ID is too long.");
			$filename = preg_replace('/template\\.html$/', "template_" . $templateID . ".html", $filename);
		}
		
	    if (substr($filename, 0, 1) != "/") {
      		$filenameCheck = $this->root."/".$filename;
    	}
		
    	// On the Development server show the template error message... on the live server show a 404 error when the template can't be found.
		if (!file_exists($filenameCheck)) {
			
			if(Constants::getDevelopmentServer()){
				exit("Template file was not found: " . $filenameCheck);
			}
			else {
				WebUtil::print404Error();
				exit();
			}
		}
		else{
			return parent::filename($filename);
		}
	}
}

