<?php

 
define("CONTEXT_START", 1);
define("CONTEXT_LENGTH", 6);
define("CONTEXT_DEPTH", 27);

define("SYMBOL_END", "");

// The following character will be substituted for unknown inputs (relative to your cardinality set). 
define("SYMBOL_UNKNOWN", "");


class Triangle {

	private $contextColumns = array();
	private static $singleton;

	function __construct(){
		
		$this->contextColumns = array(
                                               /**/
                                          "L6",     "L5", 
                                    "L4",      "L3",    "L5_G01", 
                             "L4_G01", "L3_G01",   "L4_G02", "L3_G02", 
                        "L3_G03",   "L2",     "L5_G1",   "L4_G1",  "L4_G101", 
                  "L3_G1",   "L4_G11",  "L3_G11",  "L3_G12",  "L2_G1",   "L4_G2", 
            "L3_G2",   "L3_G21",   "L2_G2",   "L3_G3",   "L2_G3",   "L2_G4",    "L1"
		);
		
		// We reverse it because we want the higher the Depth value, the stronger the CONTEXT MATCH.
		$this->contextColumns = array_reverse($this->contextColumns);
		
		if(sizeof($this->contextColumns) != CONTEXT_DEPTH){
			throw new Exception("The number of context columns does not match the CONTEXT_DEPTH constant.");
		}
		
		if(CONTEXT_LENGTH % 6 != 0){
			throw new Exception("The Context Length constant must be divisible by 6.");
		}
		
		$checkTriangle = 0;
		for($i = 1; $i <= CONTEXT_LENGTH; $i++){
			$checkTriangle += ($i+1);
		}
		
		if($checkTriangle != sizeof($this->contextColumns)){
			throw new Exception("Improper triangle formation.");
		}
	}
	
	/*
	 * return Triangle
	 */
	static function getSingleton(){
		
		if(empty(self::$singleton)){
			self::$singleton = new Triangle();
		}
		
		return self::$singleton;
	}
	
	private function validateContextDepth($depth){

		$depth = intval($depth);
		
		if( $depth < CONTEXT_START || $depth > CONTEXT_DEPTH){
			throw  new Exception("The depth value is out of range.");
		}
		
		return $depth;
	}
	
	/*  --- Not used anywhere yevalidateContextLengtht ----
	private function ($length){

		$length = intval($length);
		
		if( $length < CONTEXT_START || $length > CONTEXT_LENGTH){
			throw  new Exception("The length value is out of range.");
		}
		
		return $length;
	}
	*/
	
	/*  --- Not used anywhere yet ----
	// Returns an array of "CONTEXT COLUMN NAMES" with the "STRONGEST CONTEXT CHAIN" at the beggining of the array.
	function getContextColumnNames(){
		return $this->contextColumns;
	}
	*/
	
	// The lower the value, the lower the priority for matching within the DB.
	function getColumnNameByDepth($depth){
		
		return $this->contextColumns[$this->validateContextDepth($depth) - 1];
	}
	
	// Every Context Depth value has a "CONTEXT CHAIN LENGTH" associated with it.
	function getChainLengthFromDepth($depth){
		
		return $this->getLengthValueFromColumnName($this->getColumnNameByDepth($depth));
	}
	
	function getLengthValueFromColumnName($columnName){

		$matches = array();
		
		if(!preg_match('/^L(\d+)/', $columnName, $matches)){
			throw new Exception("Can not find a length value within the column name.");
		}
		
		return $matches[1];
	}
	
	// This is almost like getChainLengthFromDepth() except that it will tell you the number of Conext Items between the start and the last element (regardless of gaps)
	// For example the following "CONTEXT CHAIN" has an absolute length of 6 (even though the "CONTEXT CHAIN LENGTH" is 2)	EX: "F + A"
	function getAbsoluteLengthByDepth($depth){
		
		$gapPatternArr = $this->getGapPattern($depth);
		
		$counter = 0;
		$maxCounter = 0;
		foreach($gapPatternArr as $loopGapPattern){
			
			$counter++;
			
			if($loopGapPattern){
				$maxCounter = $counter;
			}
		}
		
		return $maxCounter;
	}
	
	// This will return an array with Boolean values.
	// Every "CONTEXT DEPTH" has a "CHAIN LENGTH" value, as well as a pattern of gaps.
	// A value of FALSE indicates that a "CONTEXT GAP" exists so the value will not be part of a "CONTEXT CHAIN"
	// The first "CONTEXT GAP" value will always be TRUE because the "ROOT CHAIN LINK" must exist immediately before a prediction.
	// This method will always return a consistent "array length". 
	
	// Here are some examples of what the "return arrays" would look like for the corresponding "CONTEXT DEPTHS".
	// -------------------------------------------------------
	// "CONTEXT DEPTH"		 	27	
	// "CONTEXT COLUMN NAME"	L6
	// "CONTEXT CHAIN":			F + E + D + C + B + A
	// array(true, true, true, true, true, true)
	// -------------------------------------------------------
	// "CONTEXT DEPTH"		 	22	
	// "CONTEXT COLUMN NAME"	L4_G01
	// "CONTEXT CHAIN":			F + E + C + B
	// array(true, true, false, true, true, false)
	// -------------------------------------------------------
	// "CONTEXT DEPTH"		 	2	
	// "CONTEXT COLUMN NAME"	L2_G4
	// "CONTEXT CHAIN":			F + A
	// array(true, false, false, false, false, true)
	// -------------------------------------------------------
	function getGapPattern($depth){
		
		$matches = array();
		$gapStr = "";
		
		if(preg_match('/G(\d+)$/', $this->getColumnNameByDepth($depth), $matches)){
			$gapStr = $matches[1];
		}
		
		$specifiedChainLength = $this->getChainLengthFromDepth($depth);
		
		$retArr = array();
		$retArr[0] = true;
		
		// The $retArr already has one "CHAIN LINK".
		$chainLinkCounter = 1;
		
		$inBetweenGaps = false;
		
		$gapsExtracted = 0;
		$lastGapValue = 0;
		
		for($i = 1; $i < CONTEXT_LENGTH; $i++){

			// This will pad FALSE values to the end of a short chain.
			if($chainLinkCounter >= $specifiedChainLength){
				$retArr[$i] = false;
				continue;
			}
			
			// If there are 2 gaps within a column name (such as L3_G21), we need to allow one "CHAIN LINK" to be filled between the "2" and the "1".
			if(!$inBetweenGaps){
				
				// Check and see if there is another Gap value to extract.
				if(empty($lastGapValue)){
					$lastGapValue = intval(substr($gapStr, $gapsExtracted, 1));
					$gapsExtracted++;
				}
				
				// Until we finish cycling over all of the gaps, we need to fill the array with FALSE entries.
				if(!empty($lastGapValue)){
					
					$retArr[$i] = false;
					$lastGapValue--;
	
					if($lastGapValue == 0){
						$inBetweenGaps = true;
					}
					continue;
				}
			}

			// This means that there isn't a gap at the current position.
			// Add a TRUE entry to the array and increment the size of our ChainLink counter. 
			$retArr[$i] = true;
			$chainLinkCounter++;
			$inBetweenGaps = false;
		}
		
		if(!empty($lastGapValue)){
			throw new Exception("The Gap Value is incorrect at Depth: $depth");
		}
		
		if($chainLinkCounter != $this->getChainLengthFromDepth($depth)){
			var_dump($retArr);
			exit("<br><br>The number of chain links detected by the Gap sequence ($chainLinkCounter) did not match the corresponding L value ($specifiedChainLength) at depth: $depth");
			throw new Exception("The number of chain links detected by the Gap sequence ($chainLinkCounter) did not match the corresponding L value.");
		}
		
		return $retArr;
	}
	
	/*  --- Not used anywhere yet ----
	// Returns an array of column names that have a "CONTEXT CHAIN LENGTH" matching the parameters.
	// For example, pass in 1 and this method will return array("L1").  
	// ... Pass in 2 and it will return array("L2", "L2_G1", "L2_G2", "L2_G3", "L2_G4")
	function getColumnsByContextLength($length){
		
		$length = intval($length);
		
		if($length < CONTEXT_START || $length > CONTEXT_LENGTH){
			throw new Exception("Illegal length value.");
		}
		
		$retArr = array();
		
		foreach ($this->contextColumns as $thisColumnName){
			if($this->getLengthValueFromColumnName($thisColumnName) == $length){
				$retArr[] = $thisColumnName;
			}
		}
		
		if(empty($retArr)){
			throw new Exception("Configuration error with column names.");
		}
		
		return $retArr;
	}
	*/
	
	/*  --- Not used anywhere yet ----
	// This will return an array with up to 6 values (no Gaps)
	// The highest priority matches will be seen on top (starting with L6)
	// It will also reverse the elements so that the "ROOT CHAIN LINK" appears first within the array.
	// For example, pass in an array like array(A,B,C,D,E,F) and this method will return a Hash with 1 element such as array("L6"=>"F+E+D+C+B+A").
	// Pass in an array like array(^,A,B,C) and it will return an Hash 1 element such as array("L4"=>"C+B+A+^").  Notice that the column name changed to L4 in this example.
	// The array that is passed into this method should have the "ROOT CHAIN LINK" as the last element of the array.
	// If you do not have 6 "CONTEXT ITEMS", then the first element should contain the "Begin Symbol".  It is also possible to have a 6 "CONTEXT ITEMS" with a "Begin Symbol".
	function getPerfectContextColumn(array $contextArr, $withMD5flag = true){
		
		if(sizeof($contextArr) < CONTEXT_START || sizeof($contextArr) > CONTEXT_LENGTH){
			throw new Exception("The array size is outside of the 'CONTEXT LENGTH'");
		}
		
		// Verify that the developer hasn't made any mistakes.
		for($i = CONTEXT_START; $i < sizeof($contextArr); $i++){
			if($contextArr[$i] == SYMBOL_END){
				throw new Exception("The 'End Symbol', may not be used within a context array.");
			}
		}
		
		$contextArr = array_reverse($contextArr);
		
		// Glue the Context items together by the "End Symbol".
		// That is sure not to collide with valid content.
		$retStr = implode(SYMBOL_END, $contextArr);
		
		// Optionally digest the data.
		$retStr = ($withMD5flag ? md5($retStr) : $retStr);
		
		// A Perfect column name is always an "L", followed by the context length (no gaps).
		// This make the Hash Key L1 - L6.
		return array(("L" . sizeof($contextArr)) => $retStr); 
	}
	*/
	
	// This will return a hash with 1 element.
	// The "Hash Key" represents the "Column Name" and the value represents the last element within the "Context Array". 
	function getL1(array $contextArr, $withMD5flag = true){
		
		if(sizeof($contextArr) < CONTEXT_START || sizeof($contextArr) > CONTEXT_LENGTH){
			throw new Exception("The array size is outside of the 'CONTEXT LENGTH'");
		}
		
		// The final element in the array should be our "most recent" context item.
		$retStr = end($contextArr);
		
		// A Perfect column name is always an "L", followed by the context length (no gaps).
		return array("L1" => ($withMD5flag ? md5($retStr) : $retStr)); 
	}
	
	// Pass in an array like (A, B, C, D) and it will return a signature such as "D^C^B^A"
	// It reversing the order puts the most recent context items first.  
	// If the application is used without MD5 signatures (for some reason), the sorting algorithm must give the most weight to the final element in the context array.
	// For example, whatever you said last in the converstation matters the most when I am formulating a reply.
	function getSignatureFromContextChain(array $contextArr, $withMD5flag = true){
		
		if(sizeof($contextArr) < CONTEXT_START || sizeof($contextArr) > CONTEXT_LENGTH){
			throw new Exception("The array size is outside of the 'CONTEXT LENGTH'");
		}
		
		$contextArr = array_reverse($contextArr);
		
		$signatureStr = implode(SYMBOL_END, $contextArr);
		
		return $withMD5flag ? md5($signatureStr) : $signatureStr;
	}
	
	// This will return an array with 27 values (if there are 6 Context Items).
	// If there is only 1 context item, then the only value that will be returned is a string with that item.
	// A "Split Symbol" character will separate the "CONTEXT LINKS" 
	// The highest priority matches will be seen on top (starting with L6)
	// It will also reverse the elements so that the "ROOT CHAIN LINK" appears first within the array.
	// For example, pass in an array like (A,B,C,D,E,F) and this method will return an L6 like "F^E^D^C^B^A".  
	// In the "return hash", the "keys" correspond to the name of the "database columns".
	function getC27(array &$contextArr, $withMD5flag = true){
		
		if(sizeof($contextArr) < CONTEXT_START || sizeof($contextArr) > CONTEXT_LENGTH){
			throw new Exception("The array size is outside of the 'CONTEXT LENGTH'");
		}
		
		$contextArr = array_reverse($contextArr);
		
		$retArr = array();
		
		for($i = CONTEXT_DEPTH; $i >= CONTEXT_START; $i--){
			
			// If we only have 2 Context Items... we can't make a "CONTEXT CHAIN" with 3 gaps in it.
			if($this->getAbsoluteLengthByDepth($i) > sizeof($contextArr)){
				continue;
			}
			
			$gapPattern = $this->getGapPattern($i);
			
			$tempStr = "";
			
			for($x = 0; $x < sizeof($contextArr); $x++){
				
				if($gapPattern[$x]){
					
					if(!empty($tempStr)){
						$tempStr .= SYMBOL_END;
					}
					
					$tempStr .= $contextArr[$x];
				}
			}
			
			$retArr[$this->getColumnNameByDepth($i)] = ($withMD5flag ? md5($tempStr) : $tempStr);
		}
		
		return $retArr;
	}
	
	/*  --- Not used anywhere yet ----
	// While trying to find the "STRONGEST CONTEXT CHAIN" you typically loop over a sequence of thoughts going backwards (sentences,words,letters).
	// The best possible match is to find an L6.  However, such a match is not possible to find on a Word which is the first in a sentence.
	// If you are storing words... then the first word will have 0 preceding "CONTEXT ITEMS".
	// If there is a sentence with 4 words, then the last word in the sentence will have 3 preceding "CONTEXT ITEMS".
	function checkIfDepthCanBeQueried($depth, $precedingContextItems){
		
		$columnName = $this->getColumnNameByDepth($depth);
		
		$absoluteContextLength = $this->getAbsoluteLengthByDepth($depth);
		
		if($absoluteContextLength <= ($precedingContextItems + 1)){
			return true;
		}
		else {
			return false;
		}
	}
	*/
}

