<?php


require_once 'Boot.php';

set_time_limit(1000);

$authObj = new Authenticate();

$dialogObj = new ChatDialog($authObj->getAccountID());

print "Peak Usage: " .number_format( memory_get_peak_usage(true), 0) . "<br><br>";

$thought = WebUtil::getInput("thought", FILTER_UNSAFE_RAW);
$thought = trim($thought);

// It is OK to submit the form without typing anything.
if(!empty($thought)){
	
	// This will mitigate "Cross-site Forgery" attacks.
	WebUtil::checkFormSecurityCode();
	
	$dialogObj->addThought($thought);
}

$t = new Templatex(".");

$t->set_file("root", "talk-template.html");

// Sets up variables within the nav bar.
// Also sets the "Form Security Code" Template Variable to mitigate XSS attacks.
Widgets::buildHtmlNav($t);

// Fetch the most recent thoughts within the inventor's dialog.
// If a new thought was added in this form action, it will be the first one in the array.
$dialogObj->setTailLength(200);
$dialogIDsArr = $dialogObj->getDialogStream();

// Extract an HTML block to append to within the loop below.
$t->set_block("root","ThoughtBL","ThoughtBL_out");
$t->allowVariableToContainBrackets("THOUGHT");

foreach($dialogIDsArr as $thisDialogID){
	
	// On this page, the oldest thoughts are listed at the bottom.
	// Our prediction is the most recent thing written to the DB... we we need to parse the block first.
	
	// -------- Prediction ----------
	
	$thought = $dialogObj->getContentPredicted($thisDialogID);
	
	$t->set_var("THOUGHT", $dialogObj->formatHtml($thought));
	$t->set_var("CSS_CLASS", "ReplyThought");
	
	$t->parse("ThoughtBL_out","ThoughtBL",true);
	
	
	// -------- Submission ----------
	
	$thought = $dialogObj->getContentCorrected($thisDialogID);
	
	$t->set_var("THOUGHT", $dialogObj->formatHtml($thought));
	$t->set_var("CSS_CLASS", "MyThought");
	
	$t->parse("ThoughtBL_out","ThoughtBL",true);
}

// Erase the "dialog container" if there are no thoughts inside of it.
if(empty($dialogIDsArr)){
	$t->discard_block("root", "DialogBL");
}

$t->pparse("OUT","root");

print "Peak Usage: " .number_format( memory_get_peak_usage(true), 0) . "<br>";

print "<br><u>L1 Queries</u><br>";
foreach(NeuronHelper::$queries_L1 as $loop_CharCode => $loop_Count){
	print $loop_CharCode . " : " . $loop_Count . "<br>";
}

print "<br><u>L1 Inserts</u><br>";
foreach(NeuronHelper::$inserts_L1 as $loop_CharCode => $loop_Count){
	print $loop_CharCode . " : " . $loop_Count . "<br>";
}

print "<br><u>C27 Tops</u><br>";
foreach(NeuronHelper::$queries_C27_Top as $loop_CharCode => $loop_Count){
	print $loop_CharCode . " : " . $loop_Count . "<br>";
}

print "<br><u>C27 Bottoms</u><br>";
foreach(NeuronHelper::$queries_C27_Bottom as $loop_CharCode => $loop_Count){
	print $loop_CharCode . " : " . $loop_Count . "<br>";
}




